.turbopress form ul {
    list-style: none;
    padding-left: 10px;
}

.turbopress #file {
    border: none;
}

.turbopress_buttons a img {
    width: 24px;
    border: none;
}

.turbopress_buttons a {
    text-decoration: none;
}

#turbopress_create_button_box {
    float: right;
}

.turbopress_text_button {
    border: 1px solid #CCC;
    padding: 5px;
    float: right;
    border-radius: 4px;
    background-color: #DDD;
}

.turbopress_text_button a {
    text-decoration: none;
    color: black;
    font-weight: bold;
    display: block;
}

.turbopress_text_button:hover {
    background-color: #BBB;
}

#turbopress_attachments {
    margin-top: 50px;
    width: 710px;
}

#attachments_table {
    padding-left: 10px;
}

#attachments_table a {
    color: blue;
}

#attachments_table a:hover {
    text-decoration: underline;
}

#turbopress_articles_box {
    width: 67%;
    margin-right: 3%;
}

#turbopress_rightbar {
    width: 30%;
    float: right;
}

.turbopress_tag {
    margin: 3px;
}

#turbopress_search ul {
    list-style: none;
    padding-left: 10px;
}

.turbopress_attachment_minipreview {
    height: 64px;
}

#turbopress_attachments_title {
    margin-bottom: 10px;
}

#turbopress_attachments_title span {
    font-size: 1.6em;
}

#turbopress_attachments_title em {
    margin-left: 10px;
    font-size: 11px;
}