<div xmlns:py="http://genshi.edgewall.org/" class="turbopress_article">
    <h2>
        <a href="${h.plug_url('turbopress', '/view/%s'%article.uid)}">${article.title}</a>
    </h2>
    <h4>
        <span class="small" py:if="article.is_owner(request.identity)">
            <a href="${h.plug_url('turbopress', '/edit/%s'%article.uid)}">edit</a> |
        </span>
        <span class="small">${article.author} | ${h.turbopress.format_date(article)} | ${h.turbopress.comma_separated_tags(article)}</span>
    </h4>
    <div class="turbopress_article_content">
        ${Markup(article.content)}
    </div>
</div>
